package com.fr.plugin.platform.db.transfer.dm;

import com.fr.config.FineDBDriverClassLoaderStore;
import com.fr.general.IOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.context.PluginContext;
import com.fr.plugin.observer.inner.AbstractPluginLifecycleMonitor;
import com.fr.plugin.platform.db.transfer.dm.interceptor.DBInterceptorCreationHook;
import com.fr.stable.CommonUtils;
import com.fr.stable.db.DBContext;
import com.fr.third.guava.io.ByteStreams;
import com.fr.workspace.WorkContext;

import java.io.IOException;
import java.io.InputStream;

public class DMLifecycleMonitor extends AbstractPluginLifecycleMonitor {

    @Override
    public void afterRun(PluginContext pluginContext) {
        //用配置项，而不放在afterInstall的原因：afterInstall的时候还读不到resource里的内容
        if (!DMMigrationConfig.getInstance().isInit()) {
            // 移动方言、jar包，防止重启后，外置数据库模块在插件模块之前启动，依赖不到方言和jar。
            // 此处不放在生命周期类afterInstall方法中的原因是要考虑 1，此插件有可能升级的情况  2，在afterInstall处读不到resource中的内容
            /*try {
                moveDialectClass();
                moveDriverLib();
            } catch (IOException e) {
                FineLoggerFactory.getLogger().error(e, e.getMessage());
            }*/
            DMMigrationConfig.getInstance().setInit(true);
        }

        // 产品内部逻辑实现是，注册之后的类加载器，在连接外置数据库时，都会传给druid。
        //FineDBDriverClassLoaderStore.getInstance().add(this.getClass().getClassLoader());
        DBContext.registerGlobalCreationHook(new DBInterceptorCreationHook());
    }

    @Override
    public void beforeStop(PluginContext pluginContext) {
        //FineDBDriverClassLoaderStore.getInstance().remove(this.getClass().getClassLoader());
        DBContext.unregisterGlobalCreationHook(new DBInterceptorCreationHook());
    }

    @Override
    public void beforeUninstall(PluginContext pluginContext) {
        DMMigrationConfig.getInstance().setInit(false);
    }

    private void moveDialectClass() throws IOException {
        InputStream is = IOUtils.readResource("FRDMDialect.class");
        String path = CommonUtils.pathJoin("/classes", DMDialect.class.getName().replace('.', '/') + ".class");
        if (!WorkContext.getWorkResource().exist(path)) {
            WorkContext.getWorkResource().write(path, ByteStreams.toByteArray(is));
        }
    }

    private void moveDriverLib() throws IOException {
        InputStream is = IOUtils.readResource("Dm7JdbcDriver18.jar");
        String path = "/lib/Dm7JdbcDriver18.jar";
        if (!WorkContext.getWorkResource().exist(path)) {
            WorkContext.getWorkResource().write(path, ByteStreams.toByteArray(is));
        }
    }

}